#include <stdio.h>
#include <windows.h>
#include "..\common\CSerial.h"
#include "..\common\CSocket.h"
#include "..\common\LE9x0Command.h"

int main()
{
	int ret = 0;

	// -------------------------------------------------
	// Select the communication method with LE-930R.
	// For the USB interface, open the virtual COM port.
	// For the WiFi interfaces, open a TCP socket.
	// -------------------------------------------------
#if 1
	// Open COM port (COM1)
	CSerial *comm = new CSerial();
	int comPort = 1;
	if( !comm->Open( (void*)&comPort ) )
	{
		ret = -1;
	}
#else
	// Open TCP socket
	// (IP Address = 192.168.0.10, Control port number = 10101)
	CSocket *comm = new CSocket();
	CSocket::CONNECTION connetion = { "192.168.0.10", 10101 };
	if( !comm->Open( (void*)&connetion ))
	{
		ret = -1;
	}
#endif

	if( ret < 0 )
	{
		printf( "Interface open error.\n" );
		goto error_exit;
	}

	// ------------------
	// Connect to LE-930R
	// ------------------
	FRAME Res;
	LE9x0Command *cmd = new LE9x0Command( comm );

	// Invalid connection maintenance command transmission from LE-930R
	cmd->TransmitCommandWait( CMD_CONNECT, 0x20, 0, NULL, &Res );
	if( (Res.res != RES_OK) && (Res.res != RES_ALREADY_CONNECTED) )
	{
		printf( "Connection error. [0x%02X]\n", Res.res );
		ret = -1;
		goto error_comm;
	}
	printf( "Connection OK.\n" );

	// -----------------
	// Setup for LE-930R
	// ------------------
	// Input output type
	char input[64] = "";
	LE930_TYPE type;
	printf( "Enter the output type number.\n" );
	printf( " %d : 100mV range\n", LE930_TYPE_VOLTAGE_100mV );
	printf( " %d : 10V range\n", LE930_TYPE_VOLTAGE_10V );
	printf( " %d : 4-20mA range (Use internal power supply)\n", LE930_TYPE_CURRENT_INT );
	printf( " %d : 4-20mA range (Use external power supply)\n", LE930_TYPE_CURRENT_EXT );
	printf( "> " );
	scanf_s( "%d%*[^\n]", &type );
	if( (type < 0) || (type >= LE930_TYPE_NUM) )
	{
		printf( "The input number is invalid.\n" );
		goto error_cmd;
	}

	// -------------------
	// Analog value output
	// -------------------
	float min, max, value;
	char* unit;
	switch( type )
	{
	case LE930_TYPE_VOLTAGE_100mV:
		min = -100;
		max = 100;
		unit = "mV";
		break;
	case LE930_TYPE_VOLTAGE_10V:
		min = -10;
		max = 10;
		unit = "V";
		break;
	case LE930_TYPE_CURRENT_INT:
	case LE930_TYPE_CURRENT_EXT:
		min = 0;
		max = 20;
		unit = "mA";
		break;
	}
	printf( "\nEnter the output value in the range of %.0f%s to %.0f%s.\n", min, unit, max, unit );
	printf( " (Enter ""999"" to finish.)\n" );

	// The input value is output from LE-930R.
	for(;;)
	{
		// Input output value
		printf( "> " );
		scanf_s( "%f%*[^\n]", &value );
		if( (value >= min) && (value <= max) )
		{
			// Output analog value
			DWORD ad = cmd->ConvertRealDataLE930( type, value );
			BYTE vout[3] = {0};
			vout[0] = (BYTE)type;
			vout[1] = (BYTE)(ad >> 8);
			vout[2] = (BYTE)(ad & 0xFF);
			cmd->TransmitCommandWait( CMD_DA_OUT_SET, 0x00, sizeof(vout), vout, &Res );
			if( Res.res != RES_OK )
			{
				printf( "Analog output error. [0x%02X]\n", Res.res );
				goto error_cmd;
			}
		}
		else
		{
			if( value == 999 )
				break;
			else
				printf( "Input value is invalid.\n" );
		}
	}


error_cmd:
	// ----------------------
	// Disonnect from LE-930R
	// ----------------------
	cmd->TransmitCommandWait( CMD_DISCONNECT, 0x00, 0, NULL, &Res );
	if( Res.res != RES_OK )
	{
		printf( "Disconnection error. [0x%02X]\n", Res.res );
		ret = -1;
		goto error_comm;
	}
	printf( "Disconnection OK.\n" );

error_comm:
	comm->Close();
	delete cmd;

error_exit:
	delete comm;

	return ret;
}
